@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: internal.c,v 1.51 2008/09/25 18:33:50 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: internal.c,v 1.51.2.2 2010/03/21 03:44:32 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - internal.c */
@@ -67,6 +67,13 @@ GP_MATHERR( STRUCT_EXCEPTION_P_X )
 
 #define BAD_DEFAULT default: int_error(NO_CARET, "internal error : type neither INT or CMPLX"); return;
 
+static int recursion_depth = 0;
+void
+eval_reset_after_error()
+{
+    recursion_depth = 0;
+}
+
 void
 f_push(union argument *x)
 {
@@ -124,18 +131,23 @@ f_call(union argument *x)
     struct value save_dummy;
 
     udf = x->udf_arg;
-    if (!udf->at) {		/* undefined */
+    if (!udf->at)
 	int_error(NO_CARET, "undefined function: %s", udf->udf_name);
-    }
+
     save_dummy = udf->dummy_values[0];
     (void) pop(&(udf->dummy_values[0]));
 
     if (udf->dummy_num != 1)
 	int_error(NO_CARET, "function %s requires %d variables", udf->udf_name, udf->dummy_num);
 
+    if (recursion_depth++ > STACK_DEPTH)
+	int_error(NO_CARET, "recursion depth limit exceeded");
+
     execute_at(udf->at);
     gpfree_string(&udf->dummy_values[0]);
     udf->dummy_values[0] = save_dummy;
+
+    recursion_depth--;
 }
 
 
@@ -652,6 +664,7 @@ void
 f_mult(union argument *arg)
 {
     struct value a, b, result;
+    double product;
 
     (void) arg;			/* avoid -Wunused warning */
     (void) pop(&b);
@@ -661,8 +674,11 @@ f_mult(union argument *arg)
     case INTGR:
 	switch (b.type) {
 	case INTGR:
-	    (void) Ginteger(&result, a.v.int_val *
-			    b.v.int_val);
+	    product = (double)a.v.int_val * (double)b.v.int_val;
+	    if (fabs(product) >= (double)INT_MAX)
+		(void) Gcomplex(&result, product, 0.0);
+	    else
+		(void) Ginteger(&result, a.v.int_val * b.v.int_val);
 	    break;
 	case CMPLX:
 	    (void) Gcomplex(&result, a.v.int_val *
@@ -806,7 +822,7 @@ void
 f_power(union argument *arg)
 {
     struct value a, b, result;
-    int i, t, count;
+    int i, t;
     double mag, ang;
 
     (void) arg;			/* avoid -Wunused warning */
@@ -817,19 +833,22 @@ f_power(union argument *arg)
     case INTGR:
 	switch (b.type) {
 	case INTGR:
-	    count = abs(b.v.int_val);
+	    if (a.v.int_val == 0) {
+		if (b.v.int_val < 0)
+		    undefined = TRUE;
+		(void) Ginteger(&result, b.v.int_val == 0 ? 1 : 0);
+		break;
+	    }
+	    mag = pow((double)a.v.int_val,(double)b.v.int_val);
+	    if (mag > (double)INT_MAX  ||  b.v.int_val < 0) {
+		(void) Gcomplex(&result, mag, 0.0);
+		break;
+	    }
 	    t = 1;
 	    /* this ought to use bit-masks and squares, etc */
-	    for (i = 0; i < count; i++)
+	    for (i = 0; i < b.v.int_val; i++)
 		t *= a.v.int_val;
-	    if (b.v.int_val >= 0)
-		(void) Ginteger(&result, t);
-	    else if (t != 0)
-		(void) Gcomplex(&result, 1.0 / t, 0.0);
-	    else {
-		undefined = TRUE;
-		(void) Gcomplex(&result, 0.0, 0.0);
-	    }
+	    (void) Ginteger(&result, t);
 	    break;
 	case CMPLX:
 	    if (a.v.int_val == 0) {
