@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: mouse.c,v 1.117.2.2 2009/07/05 00:07:10 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: mouse.c,v 1.117.2.4 2010/08/06 01:23:06 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - mouse.c */
@@ -261,7 +261,9 @@ alert()
     DosBeep(444, 111);
 # else
 #  ifdef HAVE_LIBREADLINE
-    rl_ding();
+#    if !defined(MISSING_RL_DING)
+        rl_ding();
+#    endif
     fflush(rl_outstream);
 #  else
     fprintf(stderr, "\a");
@@ -640,6 +642,7 @@ apply_zoom(struct t_zoom *z)
 	int i;
 	for (i=0; i<AXIS_ARRAY_SIZE; i++) {
 	    axis_array_copy[i].label = axis_array[i].label;
+	    axis_array_copy[i].ticdef.def.user = axis_array[i].ticdef.def.user;
 	}
 	memcpy(axis_array, axis_array_copy, sizeof(axis_array));
 	s[0] = '\0';	/* FIXME:  Is this better than calling replotrequest()? */
@@ -1361,9 +1364,9 @@ ChangeView(int x, int z)
     if (x) {
 	surface_rot_x += x;
 	if (surface_rot_x < 0)
-	    surface_rot_x = 0;
-	if (surface_rot_x > 180)
-	    surface_rot_x = 180;
+	    surface_rot_x += 360;
+	if (surface_rot_x > 360)
+	    surface_rot_x -= 360;
     }
     if (z) {
 	surface_rot_z += z;
@@ -1528,7 +1531,8 @@ event_buttonpress(struct gp_event_t *ge)
     start_x = mouse_x;
     start_y = mouse_y;
     zero_rot_z = surface_rot_z + 360.0 * mouse_x / term->xmax;
-    zero_rot_x = surface_rot_x - 180.0 * mouse_y / term->ymax;
+    /* zero_rot_x = surface_rot_x - 180.0 * mouse_y / term->ymax; */
+    zero_rot_x = surface_rot_x - 360.0 * mouse_y / term->ymax;
 }
 
 
@@ -1647,11 +1651,12 @@ event_motion(struct gp_event_t *ge)
 
 	if (button & (1 << 1)) {
 	    /* dragging with button 1 -> rotate */
-	    surface_rot_x = floor(0.5 + zero_rot_x + 180.0 * mouse_y / term->ymax);
+	  /*surface_rot_x = floor(0.5 + zero_rot_x + 180.0 * mouse_y / term->ymax);*/
+	    surface_rot_x = floor(0.5 + fmod(zero_rot_x + 360.0 * mouse_y / term->ymax, 360));
 	    if (surface_rot_x < 0)
-		surface_rot_x = 0;
-	    if (surface_rot_x > 180)
-		surface_rot_x = 180;
+		surface_rot_x += 360;
+	    if (surface_rot_x > 360)
+		surface_rot_x -= 360;
 	    surface_rot_z = floor(0.5 + fmod(zero_rot_z - 360.0 * mouse_x / term->xmax, 360));
 	    if (surface_rot_z < 0)
 		surface_rot_z += 360;
