@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: plot2d.c,v 1.193.2.7 2010/03/06 19:14:37 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: plot2d.c,v 1.193.2.15 2010/08/07 20:54:46 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - plot2d.c */
@@ -374,7 +374,6 @@ get_data(struct curve_points *current_plot)
 	break;
 
     case FINANCEBARS:
-    case CANDLESTICKS:
 	/* HBB 20000504: use 'z' coordinate for y-axis quantity */
 	current_plot->z_axis = current_plot->y_axis;
 	min_cols = max_cols = 5;
@@ -383,6 +382,13 @@ get_data(struct curve_points *current_plot)
 	df_axis[2] = df_axis[3] = df_axis[4] = df_axis[1];
 	break;
 
+    case CANDLESTICKS:
+	current_plot->z_axis = current_plot->y_axis;
+	min_cols = 5;
+	max_cols = 6;
+	df_axis[2] = df_axis[3] = df_axis[4] = df_axis[1];
+	break;
+
     case BOXERROR:
 	min_cols = 3;           /* HBB 20040520: fixed, was 4 */
 	max_cols = 5;
@@ -473,7 +479,7 @@ get_data(struct curve_points *current_plot)
 
 #ifdef EAM_OBJECTS
     case CIRCLES:	/* 3 + possible variable color */
-	min_cols = 3;
+        min_cols = 2;
 	max_cols = 4;
 	break;
 #endif
@@ -665,6 +671,12 @@ get_data(struct curve_points *current_plot)
 				      v[0] - boxwidth / 2, v[0] + boxwidth / 2,
 				      v[1], variable_color_value, 0.0);
 
+#ifdef EAM_OBJECTS
+	    } else if (current_plot->plot_style == CIRCLES) {
+		    /* x, y, default radius, full circle */
+		    store2d_point(current_plot, i++, v[0], v[1], v[0]+1., v[0],
+		    		  0., variable_color_value, 360.);
+#endif
 	    } else {
 		    if (current_plot->plot_style == CANDLESTICKS
 			|| current_plot->plot_style == FINANCEBARS) {
@@ -754,8 +766,9 @@ get_data(struct curve_points *current_plot)
 
 #ifdef EAM_OBJECTS
 		case CIRCLES:	/* x, y, radius */
+		    /* by default a full circle is drawn */
 		    store2d_point(current_plot, i++, v[0], v[1], v[0]-v[2], v[0]+v[2],
-		    		  v[1], variable_color_value, v[2]);
+		    		  0., variable_color_value, 360.);
 		    break;
 #endif
 		}               /*inner switch */
@@ -887,6 +900,13 @@ get_data(struct curve_points *current_plot)
 		store2d_point(current_plot, i++, v[0], v[1], v[2], v[3], v[4],
 			      v[5], 0.0);
 		break;
+
+	    case CANDLESTICKS:
+		store2d_point(current_plot, i++, v[0], v[1],
+				v[5] > 0 ? v[0]-v[5]/2. : v[0], v[0],
+				v[2], v[3], v[4]);
+		break;
+
 images:
             case RGBA_IMAGE:  /* x_cent y_cent red green blue alpha */
             case RGBIMAGE:    /* x_cent y_cent red green blue */
@@ -907,6 +927,10 @@ images:
 
     }                           /*while */
 
+    /* This removes extra point caused by blank lines after data. */
+    if (current_plot->points[i-1].type == UNDEFINED)
+	i--;
+
     current_plot->p_count = i;
     cp_extend(current_plot, i); /* shrink to fit */
 
@@ -2292,7 +2316,10 @@ eval_plots()
 			    /* If non-para, it must be INRANGE */
 			    /* logscale ? log(x) : x */
 			    this_plot->points[i].x = t;
-			    if (boxwidth >= 0 && boxwidth_is_absolute) {
+
+			    /* For boxes [only] check use of boxwidth */
+			    if ((this_plot->plot_style == BOXES)
+			    &&  (boxwidth >= 0 && boxwidth_is_absolute)) {
 				double xlow, xhigh;
 				int dmy_type = INRANGE;
 				this_plot->points[i].z = 0;
@@ -2310,6 +2337,13 @@ eval_plots()
 				STORE_WITH_LOG_AND_UPDATE_RANGE( this_plot->points[i].xhigh, xhigh, dmy_type, x_axis,
 								    this_plot->noautoscale, NOOP, NOOP );
 			    }
+			    /* Fill in additional fields needed to draw a circle */
+			    if (this_plot->plot_style == CIRCLES) {
+				this_plot->points[i].xlow = t+1.;
+				this_plot->points[i].ylow = 0;
+				this_plot->points[i].xhigh = 360;
+			    }
+
 			    STORE_WITH_LOG_AND_UPDATE_RANGE(this_plot->points[i].y, temp, this_plot->points[i].type, in_parametric ? x_axis : y_axis,
 			    				    this_plot->noautoscale, NOOP, goto come_here_if_undefined);
 
@@ -2437,24 +2471,19 @@ eval_plots()
 	fill_gpval_string("GPVAL_LAST_PLOT", replot_line);
     }
 
-    if (table_mode)
+    if (table_mode) {
 	print_table(first_plot, plot_num);
-    else {
-	START_LEAK_CHECK();     /* check for memory leaks in this routine */
 
+    } else {
 	/* do_plot now uses axis_array[] */
 	do_plot(first_plot, plot_num);
 
-	END_LEAK_CHECK();
-
 	/* after do_plot(), axis_array[].min and .max
 	 * contain the plotting range actually used (rounded
 	 * to tic marks, not only the min/max data values)
 	 *  --> save them now for writeback if requested
 	 */
 	SAVE_WRITEBACK_ALL_AXES;
-	/* update GPVAL_ variables available to user */
-	update_gpval_variables(1);
 
 #ifdef VOLATILE_REFRESH
 	/* Mark these plots as safe for quick refresh */
@@ -2463,6 +2492,9 @@ eval_plots()
 #endif
     }
 
+    /* update GPVAL_ variables available to user */
+    update_gpval_variables(1);
+
 }                               /* eval_plots */
 
 
@@ -2480,9 +2512,7 @@ parametric_fixup(struct curve_points *start_plot, int *plot_num)
 {
     struct curve_points *xp, *new_list = NULL, *free_list = NULL;
     struct curve_points **last_pointer = &new_list;
-    size_t tlen;
     int i, curve;
-    char *new_title;
 
     /*
      * Ok, go through all the plots and move FUNC types together.  Note: this
@@ -2565,16 +2595,6 @@ parametric_fixup(struct curve_points *start_plot, int *plot_num)
 		}
 	    }
 
-	    /* Ok, fix up the title to include both the xp and yp plots. */
-	    if (xp->title && xp->title[0] != '\0' && yp->title) {
-		tlen = strlen(yp->title) + strlen(xp->title) + 3;
-		new_title = gp_alloc(tlen, "string");
-		strcpy(new_title, xp->title);
-		strcat(new_title, ", ");
-		strcat(new_title, yp->title);
-		free(yp->title);
-		yp->title = new_title;
-	    }
 	    /* move xp to head of free list */
 	    xp->next = free_list;
 	    free_list = xp;
