@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: pm3d.c,v 1.66.2.6 2009/06/19 00:01:00 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: pm3d.c,v 1.77 2009/03/26 00:49:16 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - pm3d.c */
@@ -43,9 +43,6 @@ pm3d_struct pm3d = {
     PM3D_SCANS_AUTOMATIC,	/* scans direction is determined automatically */
     PM3D_CLIP_4IN,		/* clipping: all 4 points in ranges */
     0,				/* no pm3d hidden3d is drawn */
-#if PM3D_HAVE_SOLID
-    0,				/* solid (off by default, that means `transparent') */
-#endif /* PM3D_HAVE_SOLID */
     PM3D_EXPLICIT,		/* implicit */
     PM3D_WHICHCORNER_MEAN,	/* color from which corner(s) */
     1,				/* interpolate along scanline */
@@ -274,10 +271,8 @@ pm3d_rearrange_part(
 	}
     }
 
-#if 0
-    fprintf(stderr, "(pm3d_rearrange_part) invert       = %d\n", *invert);
-    fprintf(stderr, "(pm3d_rearrange_part) invert_order = %d\n", invert_order);
-#endif
+    FPRINTF((stderr, "(pm3d_rearrange_part) invert       = %d\n", *invert));
+    FPRINTF((stderr, "(pm3d_rearrange_part) invert_order = %d\n", invert_order));
 
     for (scanA = src, scan = len - 1, i = 0; scan >= 0; --scan, i++) {
 	if (pm3d.direction == PM3D_SCANS_AUTOMATIC) {
@@ -427,6 +422,7 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
     double avgC, gray;
     double cb1, cb2, cb3, cb4;
     gpdPoint corners[4];
+    int interp_i, interp_j;
 #ifdef EXTENDED_COLOR_SPECS
     gpiPoint icorners[4];
 #endif
@@ -460,6 +456,50 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
 
     pm3d_rearrange_scan_array(this_plot, &scan_array, &scan_array_n, &invert, (struct iso_curve ***) 0, (int *) 0, (int *) 0);
 
+    interp_i = pm3d.interp_i;
+    interp_j = pm3d.interp_j;
+
+    if (interp_i <= 0 || interp_j <= 0) {
+	/* Number of interpolations will be determined from desired number of points.
+	   Search for number of scans and maximal number of points in a scan for points
+	   which will be plotted (INRANGE). Then set interp_i,j so that number of points
+	   will be a bit larger than |interp_i,j|.
+	   If (interp_i,j==0) => set this number of points according to DEFAULT_OPTIMAL_NB_POINTS.
+	   Ideally this should be comparable to the resulution of the output device, which
+	   can hardly by done at this high level instead of the driver level.
+   	*/
+	#define DEFAULT_OPTIMAL_NB_POINTS 200
+	int max_scan_pts = 0;
+	int max_scans = 0;
+	int pts;
+	for (scan = 0; scan < this_plot->num_iso_read - 1; scan++) {
+	    scanA = scan_array[scan];
+	    pointsA = scanA->points;
+	    pts = 0;
+	    for (j=0; j<scanA->p_count; j++)
+		if (pointsA[j].type == INRANGE) pts++;
+	    if (pts > 0) {
+		max_scan_pts = GPMAX(max_scan_pts, pts);
+		max_scans++;
+	    }
+	}
+
+	if (interp_i <= 0) {
+	    ii = (interp_i == 0) ? DEFAULT_OPTIMAL_NB_POINTS : -interp_i;
+	    interp_i = floor(ii / max_scan_pts) + 1;
+	}
+	if (interp_j <= 0) {
+	    ii = (interp_j == 0) ? DEFAULT_OPTIMAL_NB_POINTS : -interp_j;
+	    interp_j = floor(ii / max_scans) + 1;
+	}
+#if 0
+	fprintf(stderr, "pm3d.interp_i=%i\t pm3d.interp_j=%i\n", pm3d.interp_i, pm3d.interp_j);
+	fprintf(stderr, "INRANGE: max_scans=%i  max_scan_pts=%i\n", max_scans, max_scan_pts);
+	fprintf(stderr, "seting interp_i=%i\t interp_j=%i => there will be %i and %i points\n",
+		interp_i, interp_j, interp_i*max_scan_pts, interp_j*max_scans);
+#endif
+    }
+
     if (pm3d.direction == PM3D_DEPTH) {
 
 	for (scan = 0; scan < this_plot->num_iso_read - 1; scan++) {
@@ -474,8 +514,8 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
 		allocated_quadrangles += GPMAX(scanA->p_count, scanB->p_count) - 1;
 	    }
 	}
-	allocated_quadrangles *= (pm3d.interp_i > 1) ? pm3d.interp_i : 1;
-	allocated_quadrangles *= (pm3d.interp_j > 1) ? pm3d.interp_j : 1;
+	allocated_quadrangles *= (interp_i > 1) ? interp_i : 1;
+	allocated_quadrangles *= (interp_j > 1) ? interp_j : 1;
 	quadrangles = (quadrangle*)gp_realloc(quadrangles, allocated_quadrangles * sizeof (quadrangle), "pm3d_plot->quadrangles");
 	/* DEBUG: fprintf(stderr, "allocated_quadrangles = %d\n", allocated_quadrangles); */
     }
@@ -507,10 +547,10 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
      * if bilinear interpolation is enabled, allocate memory for the
      * interpolated points here
      */
-    if (pm3d.interp_i > 1 || pm3d.interp_j > 1) {
-	bl_point = (gpdPoint **)gp_alloc(sizeof(gpdPoint*) * (pm3d.interp_i+1), "bl-interp along scan");
-	for (i1 = 0; i1 <= pm3d.interp_i; i1++)
-	    bl_point[i1] = (gpdPoint *)gp_alloc(sizeof(gpdPoint) * (pm3d.interp_j+1), "bl-interp between scan");
+    if (interp_i > 1 || interp_j > 1) {
+	bl_point = (gpdPoint **)gp_alloc(sizeof(gpdPoint*) * (interp_i+1), "bl-interp along scan");
+	for (i1 = 0; i1 <= interp_i; i1++)
+	    bl_point[i1] = (gpdPoint *)gp_alloc(sizeof(gpdPoint) * (interp_j+1), "bl-interp between scan");
     }
 
     /*
@@ -523,9 +563,10 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
     for (scan = 0; scan < this_plot->num_iso_read - 1; scan++) {
 	scanA = scan_array[scan];
 	scanB = scan_array[scan + 1];
-#if 0
-	printf("\n#IsoCurveA = scan nb %d has %d points   ScanB has %d points\n", scan, scanA->p_count, scanB->p_count);
-#endif
+
+	FPRINTF((stderr,"\n#IsoCurveA = scan nb %d has %d points   ScanB has %d points\n",
+		scan, scanA->p_count, scanB->p_count));
+
 	pointsA = scanA->points;
 	pointsB = scanB->points;
 	/* if the number of points in both scans is not the same, then the
@@ -587,9 +628,10 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
 		i1 = ii1;
 		ii1 = itmp;
 	    }
-#if 0
-	    fprintf(stderr,"j=%i:  i=%i i1=%i  [%i]   ii=%i ii1=%i  [%i]\n",j,i,i1,scanA->p_count,ii,ii1,scanB->p_count);
-#endif
+
+	    FPRINTF((stderr,"j=%i:  i=%i i1=%i  [%i]   ii=%i ii1=%i  [%i]\n",
+	    		j,i,i1,scanA->p_count,ii,ii1,scanB->p_count));
+
 	    /* choose the clipping method */
 	    if (pm3d.clip == PM3D_CLIP_4IN) {
 		/* (1) all 4 points of the quadrangle must be in x and y range */
@@ -607,7 +649,7 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
 		    continue;
 	    }
 
-	    if ((pm3d.interp_i <= 1 && pm3d.interp_j <= 1) || pm3d.direction == PM3D_DEPTH) {
+	    if ((interp_i <= 1 && interp_j <= 1) || pm3d.direction == PM3D_DEPTH) {
 #ifdef EXTENDED_COLOR_SPECS
 	      if (!supply_extended_color_specs) {
 #endif
@@ -642,14 +684,12 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
 		}
 		/* transform z value to gray, i.e. to interval [0,1] */
 		gray = cb2gray(avgC);
-#if 0
+
 		/* print the quadrangle with the given color */
-		printf("averageColor %g\tgray=%g\tM %g %g L %g %g L %g %g L %g %g\n",
-		       avgC,
-		       gray,
-		       pointsA[i].x, pointsA[i].y, pointsB[ii].x, pointsB[ii].y,
-		       pointsB[ii1].x, pointsB[ii1].y, pointsA[i1].x, pointsA[i1].y);
-#endif
+		FPRINTF((stderr, "averageColor %g\tgray=%g\tM %g %g L %g %g L %g %g L %g %g\n",
+		       avgC, gray, pointsA[i].x, pointsA[i].y, pointsB[ii].x, pointsB[ii].y,
+		       pointsB[ii1].x, pointsB[ii1].y, pointsA[i1].x, pointsA[i1].y));
+
 		/* set the color */
 		if (pm3d.direction != PM3D_DEPTH)
 		    set_color(gray);
@@ -667,7 +707,7 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
 	    corners[3].x = pointsA[i1].x;
 	    corners[3].y = pointsA[i1].y;
 
-	    if ( pm3d.interp_i > 1 || pm3d.interp_j > 1 || at_which_z == PM3D_AT_SURFACE) {
+	    if (interp_i > 1 || interp_j > 1 || at_which_z == PM3D_AT_SURFACE) {
 		/* always supply the z value if
 		 * EXTENDED_COLOR_SPECS is defined
 		 */
@@ -715,56 +755,56 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
 	    } else
     		filled_quadrangle(corners, icorners);
 #else
-	    if (pm3d.interp_i > 1 || pm3d.interp_j > 1) {
+	    if (interp_i > 1 || interp_j > 1) {
 		/* Interpolation is enabled.
 		 * interp_i is the # of points along scan lines
 		 * interp_j is the # of points between scan lines
 		 * Algorithm is to first sample i points along the scan lines
 		 * defined by corners[3],corners[0] and corners[2],corners[1]. */
 		int j1;
-		for (i1 = 0; i1 <= pm3d.interp_i; i1++) {
+		for (i1 = 0; i1 <= interp_i; i1++) {
 		    bl_point[i1][0].x = 
-			((corners[3].x - corners[0].x) / pm3d.interp_i) * i1 + corners[0].x;
-		    bl_point[i1][pm3d.interp_j].x = 
-			((corners[2].x - corners[1].x) / pm3d.interp_i) * i1 + corners[1].x;
+			((corners[3].x - corners[0].x) / interp_i) * i1 + corners[0].x;
+		    bl_point[i1][interp_j].x = 
+			((corners[2].x - corners[1].x) / interp_i) * i1 + corners[1].x;
 		    bl_point[i1][0].y =
-			((corners[3].y - corners[0].y) / pm3d.interp_i) * i1 + corners[0].y;
-		    bl_point[i1][pm3d.interp_j].y =
-			((corners[2].y - corners[1].y) / pm3d.interp_i) * i1 + corners[1].y;
+			((corners[3].y - corners[0].y) / interp_i) * i1 + corners[0].y;
+		    bl_point[i1][interp_j].y =
+			((corners[2].y - corners[1].y) / interp_i) * i1 + corners[1].y;
 		    bl_point[i1][0].z =
-			((corners[3].z - corners[0].z) / pm3d.interp_i) * i1 + corners[0].z;
-		    bl_point[i1][pm3d.interp_j].z =
-			((corners[2].z - corners[1].z) / pm3d.interp_i) * i1 + corners[1].z;
+			((corners[3].z - corners[0].z) / interp_i) * i1 + corners[0].z;
+		    bl_point[i1][interp_j].z =
+			((corners[2].z - corners[1].z) / interp_i) * i1 + corners[1].z;
 		    if (color_from_column) {
 			bl_point[i1][0].c =
-			    ((corners[3].c - corners[0].c) / pm3d.interp_i) * i1 + corners[0].c;
-			bl_point[i1][pm3d.interp_j].c =
-			    ((corners[2].c - corners[1].c) / pm3d.interp_i) * i1 + corners[1].c;
+			    ((corners[3].c - corners[0].c) / interp_i) * i1 + corners[0].c;
+			bl_point[i1][interp_j].c =
+			    ((corners[2].c - corners[1].c) / interp_i) * i1 + corners[1].c;
 		    }
 		    /* Next we sample j points between each of the new points
 		     * created in the previous step (this samples between
 		     * scan lines) in the same manner. */
-		    for (j1 = 1; j1 < pm3d.interp_j; j1++) {
+		    for (j1 = 1; j1 < interp_j; j1++) {
 			bl_point[i1][j1].x =
-			    ((bl_point[i1][pm3d.interp_j].x - bl_point[i1][0].x) / pm3d.interp_j) *
+			    ((bl_point[i1][interp_j].x - bl_point[i1][0].x) / interp_j) *
 				j1 + bl_point[i1][0].x;
 			bl_point[i1][j1].y =
-			    ((bl_point[i1][pm3d.interp_j].y - bl_point[i1][0].y) / pm3d.interp_j) *
+			    ((bl_point[i1][interp_j].y - bl_point[i1][0].y) / interp_j) *
 				j1 + bl_point[i1][0].y;
 			bl_point[i1][j1].z =
-			    ((bl_point[i1][pm3d.interp_j].z - bl_point[i1][0].z) / pm3d.interp_j) *
+			    ((bl_point[i1][interp_j].z - bl_point[i1][0].z) / interp_j) *
 				j1 + bl_point[i1][0].z;
 			if (color_from_column)
 			    bl_point[i1][j1].c =
-				((bl_point[i1][pm3d.interp_j].c - bl_point[i1][0].c) / pm3d.interp_j) *
+				((bl_point[i1][interp_j].c - bl_point[i1][0].c) / interp_j) *
 				    j1 + bl_point[i1][0].c;
 		    }
 		}
 		/* Once all points are created, move them into an appropriate
 		 * structure and call set_color on each to retrieve the
 		 * correct color mapping for this new sub-sampled quadrangle. */
-		for (i1 = 0; i1 < pm3d.interp_i; i1++) {
-		    for (j1 = 0; j1 < pm3d.interp_j; j1++) {
+		for (i1 = 0; i1 < interp_i; i1++) {
+		    for (j1 = 0; j1 < interp_j; j1++) {
 			corners[0].x = bl_point[i1][j1].x;
 			corners[0].y = bl_point[i1][j1].y;
 			corners[0].z = bl_point[i1][j1].z;
@@ -777,19 +817,17 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
 			corners[3].x = bl_point[i1][j1+1].x;
 			corners[3].y = bl_point[i1][j1+1].y;
 			corners[3].z = bl_point[i1][j1+1].z;
- 			if (color_from_column) {
- 			    corners[0].c = bl_point[i1][j1].c;
- 			    corners[1].c = bl_point[i1+1][j1].c;
- 			    corners[2].c = bl_point[i1+1][j1+1].c;
- 			    corners[3].c = bl_point[i1][j1+1].c;
- 			}
-#if 0
-			printf("(%g,%g),(%g,%g),(%g,%g),(%g,%g)\n",
-			    corners[0].x, corners[0].y,
-			    corners[1].x, corners[1].y,
-			    corners[2].x, corners[2].y,
-			    corners[3].x, corners[3].y);
-#endif
+			if (color_from_column) {
+			    corners[0].c = bl_point[i1][j1].c;
+			    corners[1].c = bl_point[i1+1][j1].c;
+			    corners[2].c = bl_point[i1+1][j1+1].c;
+			    corners[3].c = bl_point[i1][j1+1].c;
+			}
+
+			FPRINTF((stderr,"(%g,%g),(%g,%g),(%g,%g),(%g,%g)\n",
+			    corners[0].x, corners[0].y, corners[1].x, corners[1].y,
+			    corners[2].x, corners[2].y, corners[3].x, corners[3].y));
+
 			/* If the colors are given separately, we already loaded them above */
 			if (!color_from_column) {
 			    cb1 = z2cb(corners[0].z);
@@ -828,7 +866,7 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
 			}
 		    }
 		}
-	    } else { /* thus (pm3d.interp_i == 1 && pm3d.interp_j == 1) */
+	    } else { /* thus (interp_i == 1 && interp_j == 1) */
 		if (pm3d.direction != PM3D_DEPTH) {
 		    filled_quadrangle(corners);
 		} else {
@@ -844,7 +882,7 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
     } /* loop over scans */
 
     if (bl_point) {
-	for (i1 = 0; i1 <= pm3d.interp_i; i1++)
+	for (i1 = 0; i1 <= interp_i; i1++)
 	    free(bl_point[i1]);
 	free(bl_point);
     }
@@ -914,9 +952,6 @@ pm3d_reset()
     pm3d.direction = PM3D_SCANS_AUTOMATIC;
     pm3d.clip = PM3D_CLIP_4IN;
     pm3d.hidden3d_tag = 0;
-#if PM3D_HAVE_SOLID
-    pm3d.solid = 0;
-#endif /* PM3D_HAVE_SOLID */
     pm3d.implicit = PM3D_EXPLICIT;
     pm3d.which_corner_color = PM3D_WHICHCORNER_MEAN;
     pm3d.interp_i = 1;
@@ -1028,10 +1063,8 @@ set_plot_with_palette(int plot_num, int plot_mode)
     /* Check 2D plots */
     if (plot_mode == MODE_PLOT) {
 	while (this_2dplot) {
-#ifdef WITH_IMAGE
 	    if (this_2dplot->plot_style == IMAGE)
 		return;
-#endif
 	    if (this_2dplot->lp_properties.use_palette
 	    &&  this_2dplot->lp_properties.pm3d_color.type > TC_RGB)
 		return;
@@ -1048,10 +1081,8 @@ set_plot_with_palette(int plot_num, int plot_mode)
 	while (surface++ < plot_num) {
 	    if (this_3dplot->plot_style == PM3DSURFACE)
 		return;
-#ifdef WITH_IMAGE
 	    if (this_3dplot->plot_style == IMAGE)
 		return;
-#endif
 	    if (this_3dplot->lp_properties.use_palette) {
 	        int type = this_3dplot->lp_properties.pm3d_color.type;
 		if (type == TC_LT || type == TC_LINESTYLE || type == TC_RGB)
@@ -1061,11 +1092,9 @@ set_plot_with_palette(int plot_num, int plot_mode)
 		    /* TC_DEFAULT: splot x with line|lp|dot palette */
 		    return;
 	    }
-#ifdef EAM_DATASTRINGS
 	    if (this_3dplot->labels &&
 		this_3dplot->labels->textcolor.type >= TC_CB)
 		return;
-#endif
 	    this_3dplot = this_3dplot->next_sp;
 	}
     }
