@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: show.c,v 1.227.2.3 2009/12/20 03:54:42 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: show.c,v 1.227.2.7 2010/08/09 21:38:05 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - show.c */
@@ -871,6 +871,9 @@ show_version(FILE *fp)
 #else
 		"LIBREADLINE  "
 #endif
+#if defined(HAVE_LIBREADLINE) && defined(MISSING_RL_TILDE_EXPANSION)
+		"+READLINE_IS_REALLY_EDITLINE  "
+#endif
 #ifdef GNUPLOT_HISTORY
 		"+"
 #else
@@ -985,7 +988,7 @@ show_version(FILE *fp)
 	    "";
 
 	    sprintf(compile_options, "\
-%s%s%s%s\n\
+%s%s\n%s%s\n\
 %s%s\n\
 %s%s%s%s%s%s\n%s\n",
 		    rdline, gnu_rdline, compatibility, binary_files,
@@ -1472,6 +1475,10 @@ show_style()
 	fprintf(stderr, ", lw %.1f ", default_rectangle.lp_properties.l_width);
 	fprintf(stderr, ", fillstyle");
 	save_fillstyle(stderr, &default_rectangle.fillstyle);
+    case SHOW_STYLE_CIRCLE:
+	fprintf(stderr, "\tCircle style has default radius ");
+	show_position(&default_circle.o.circle.extent);
+
 	c_token++;
 #endif
 	break;
@@ -2355,11 +2362,9 @@ show_fit()
 {
     SHOW_ALL_NL;
 
-#ifdef GP_FIT_ERRVARS
     fprintf(stderr, "\
 \tfit will%s place parameter errors in variables\n",
 	    fit_errorvariables ? "" : " not");
-#endif /* GP_FIT_ERRVARS */
 
     if (fitlogfile != NULL) {
         fprintf(stderr, "\
