@@ -1,5 +1,5 @@
 /*
- * $Id: tables.h,v 1.68.2.1 2009/12/20 03:54:42 sfeam Exp $
+ * $Id: tables.h,v 1.68.2.2 2010/06/26 06:43:47 sfeam Exp $
  */
 
 /* GNUPLOT - tables.h */
@@ -181,7 +181,8 @@ enum test_id {
 enum show_style_id {
     SHOW_STYLE_INVALID,
     SHOW_STYLE_DATA, SHOW_STYLE_FUNCTION, SHOW_STYLE_LINE,
-    SHOW_STYLE_FILLING, SHOW_STYLE_ARROW, SHOW_STYLE_RECTANGLE,
+    SHOW_STYLE_FILLING, SHOW_STYLE_ARROW, 
+    SHOW_STYLE_CIRCLE, SHOW_STYLE_RECTANGLE,
     SHOW_STYLE_INCREMENT, SHOW_STYLE_HISTOGRAM
 };
 
