@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: term.c,v 1.184.2.7 2010/02/18 04:34:49 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: term.c,v 1.184.2.9 2010/07/09 17:27:17 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - term.c */
@@ -878,6 +878,7 @@ term_apply_lp_properties(struct lp_style_type *lp)
      *  this function by explicitly issuing additional '(*term)(...)'
      *  commands.
      */
+    int lt = lp->l_type;
 
     if (lp->pointflag) {
 	/* change points, too
@@ -891,13 +892,17 @@ term_apply_lp_properties(struct lp_style_type *lp)
 	    (*term->pointsize) (lp->p_size);
     }
     /*  _first_ set the line width, _then_ set the line type !
-
      *  The linetype might depend on the linewidth in some terminals.
      */
     (*term->linewidth) (lp->l_width);
 
+    /* FIXME: This shouldn't happen, because the higher level code */
+    /* should have made some decision about color before this. But */
+    /* better to draw in black than not to draw at all.            */
+    if (lt <= LT_COLORFROMCOLUMN) lt = LT_BLACK;
+
     /* Apply "linetype", which can include both color and dot/dash */
-    (*term->linetype) (lp->l_type);
+    (*term->linetype) (lt);
     /* Possibly override the linetype color with a fancier colorspec */
     if (lp->use_palette)
 	apply_pm3dcolor(&lp->pm3d_color, term);
@@ -2839,7 +2844,9 @@ int len;
 	FPRINTF((stderr,"Estimating length %d height %g for enhanced text string \"%s\"\n",
 		len, (double)(term->ymax)/10., text));
 	term = tsave;
-    } else
+    } else if (encoding == S_ENC_UTF8)
+	len = strlen_utf8(text);
+    else
 #endif
 	len = strlen(text);
 
