@@ -40,8 +40,8 @@ static char *RCSid() { return RCSid("$Id: version.c,v 1.89.2.8 2010/03/05 06:52:
 
 
 const char gnuplot_version[] = "4.4";
-const char gnuplot_patchlevel[] = "0";
-const char gnuplot_date[] = "March 2010";
+const char gnuplot_patchlevel[] = "2";
+const char gnuplot_date[] = "Wed Sep 22 12:10:34 PDT 2010";
 const char gnuplot_copyright[] = "Copyright (C) 1986-1993, 1998, 2004, 2007-2010";
 
 char *compile_options = (void *)0;	/* Will be loaded at runtime */
